{****************************
F1DBEditButton.
Simple Edit Control with a Button
Revision history:
1.0.0: Initial release
1.0.1: Property Direct Input:Boolean: Wether user can edit the DBEdit directly or not
       Property ButtonShortCut: for keyboard savvy user.
*****************************}
unit F1DBEditButton;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ExtCtrls
  , DbCtrls, DB, Buttons,StdCtrls,LCLType;

type

  { TF1DBEditButton }

  TF1DBEditButton = class(TPanel)
  private
    { Private declarations }
    FDBEdit: TDBEdit ;
    FButton : TBitBtn;
    FButtonClickEvent : TNotifyEvent;
    FChangeEvent : TNotifyEvent;
    FPasswordChar: Char;
    FRealMask: string;
    FButtonWidth:Integer;
    FDirectInput:Boolean;
    FButtonShortCut : TShortCut;
    function GetButtonWidth:Integer;
    function GetCharCase: TEditCharCase;
    function GetCustomEditMask: Boolean;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetReadOnly: Boolean;
    procedure SetButtonShortCut(AValue: TShortCut);
    procedure SetButtonWidth(AValue: Integer);
    procedure SetCharCase(AValue: TEditCharCase);
    procedure SetCustomEditMask(AValue: Boolean);
    procedure SetDataField(AValue: string);
    procedure SetDataSource(AValue: TDataSource);
    procedure SetDirectInput(AValue: Boolean);
    procedure SetMask(AValue: string);
    procedure SetPasswordChar(AValue: Char);
    procedure SetReadOnly(AValue: Boolean);
    function GetGlyph:TBitmap;
    procedure SetGlyph( Bitmap:TBitmap);
  protected
    { Protected declarations }
    procedure FButtonClick(Sender: TObject);
    procedure FDbEditChange(Sender:TObject);
    procedure FDbEditKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure PanelSizeChange(Sender:TObject);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;

    // Exposed public property TDBEdit
    function ExecuteAction(AAction: TBasicAction): Boolean;
    function UpdateAction(AAction: TBasicAction): Boolean;
    property Field: TField read GetField;

  published
    { Published declarations }

    // New Property
    property ButtonWidth:Integer read GetButtonWidth write SetButtonWidth default 24;
    property DirectInput:Boolean read FDirectInput write SetDirectInput;
    property ButtonShortCut : TShortCut read FButtonShortCut write SetButtonShortCut stored;

    // Exposed property TBitBtn
    property Glyph:TBitmap read GetGlyph write SetGlyph;

    // Exposed property TDBEdit
    property CustomEditMask: Boolean read GetCustomEditMask write SetCustomEditMask default False;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property BorderSpacing;
    property BorderStyle;
    property CharCase: TEditCharCase read GetCharCase write SetCharCase;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property EditMask: string read FRealMask write SetMask;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar: Char read FPasswordChar write SetPasswordChar default #0;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;

    property OnChange:TNotifyEvent read FChangeEvent write FChangeEvent;
    property OnButtonClick:TNotifyEvent read FButtonClickEvent write FButtonClickEvent;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEditingDone;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnStartDrag;
    property OnUTF8KeyPress;
  end;

procedure Register;

implementation

function TF1DBEditButton.GetField: TField;
begin
  Result := Self.FDBEdit.Field;
end;

function TF1DBEditButton.GetReadOnly: Boolean;
begin
  Result := Self.FDBEdit.ReadOnly;
end;

procedure TF1DBEditButton.SetButtonShortCut(AValue: TShortCut);
begin
  if FButtonShortCut=AValue then Exit;
  Self.FButtonShortCut:=AValue;
end;

procedure TF1DBEditButton.SetButtonWidth(AValue: Integer);
begin
  if Self.FButtonWidth=AValue then Exit;
  Self.FDBEdit.Width:=Self.Width - AValue - 1;
  //Self.FButton.Width:=AValue;
end;

procedure TF1DBEditButton.SetCharCase(AValue: TEditCharCase);
begin
  Self.FDBEdit.CharCase:=AValue;
end;

function TF1DBEditButton.GetCustomEditMask: Boolean;
begin
  Result := Self.FDBEdit.CustomEditMask;
end;

function TF1DBEditButton.GetButtonWidth: Integer;
begin
  Result := Self.FButtonWidth;
end;

function TF1DBEditButton.GetCharCase: TEditCharCase;
begin
  Result := Self.FDBEdit.CharCase;
end;

function TF1DBEditButton.GetDataField: string;
begin
  Result := Self.FDBEdit.DataField;
end;

function TF1DBEditButton.GetDataSource: TDataSource;
begin
  Result := Self.FDBEdit.DataSource;
end;

procedure TF1DBEditButton.SetCustomEditMask(AValue: Boolean);
begin
  Self.FDBEdit.CustomEditMask:=AValue;
end;

procedure TF1DBEditButton.SetDataField(AValue: string);
begin
  Self.FDBEdit.DataField:=AValue;
end;

procedure TF1DBEditButton.SetDataSource(AValue: TDataSource);
begin
  Self.FDBEdit.DataSource := AValue;
end;

procedure TF1DBEditButton.SetDirectInput(AValue: Boolean);
begin
  if FDirectInput=AValue then Exit;
  FDirectInput:=AValue;
  Self.FDBEdit.ReadOnly:=not AValue;
end;

procedure TF1DBEditButton.SetMask(AValue: string);
begin
  if FRealMask=AValue then Exit;
  FRealMask:=AValue;
end;

procedure TF1DBEditButton.SetPasswordChar(AValue: Char);
begin
  if FPasswordChar=AValue then Exit;
  FPasswordChar:=AValue;
end;

procedure TF1DBEditButton.SetReadOnly(AValue: Boolean);
begin
  Self.FDBEdit.ReadOnly:=AValue;
end;

procedure TF1DBEditButton.FButtonClick(Sender: TObject);
begin
  Self.FDBEdit.SetFocus;
  if Assigned( Self.FButtonClickEvent) then begin
    Self.FButtonClickEvent(Self);
  end;
end;

procedure TF1DBEditButton.FDbEditChange(Sender: TObject);
begin
  if Assigned( Self.FChangeEvent) then begin
    Self.FChangeEvent(Self);
  end;
  ;
end;

procedure TF1DBEditButton.FDbEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Assigned(Self.OnKeyDown) then begin
     Self.OnKeyDown(Sender,Key,Shift);
  end;
   if LCLType.KeyToShortCut(Key,Shift) = Self.FButtonShortCut then begin
      Self.FButton.Click;
   end;
end;

procedure TF1DBEditButton.PanelSizeChange(Sender: TObject);
begin
  Self.FDBEdit.Width:=Self.Width - Self.FButtonWidth - 1 ;
end;

function TF1DBEditButton.GetGlyph: TBitmap;
begin
  Result := Self.FButton.Glyph;
end;

procedure TF1DBEditButton.SetGlyph(Bitmap: TBitmap);
begin
 Self.FButton.Glyph := Bitmap;
end;

{ TF1DBEditButton }
constructor TF1DBEditButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.Width:=105;
  Self.FButtonWidth:=24;
  Self.FDBEdit := TDBEdit.Create(Self);
  Self.FDBEdit.Parent := Self;
  Self.Height:=Self.FDBEdit.Height;
  Self.FDBEdit.Top:=0;
  Self.FDBEdit.Left:=0;
  Self.FDBEdit.Align:=alLeft;
  Self.FDBEdit.Anchors := Self.FDBEdit.Anchors + [akRight];
  Self.FDBEdit.ParentFont:=True;
  Self.FDBEdit.OnChange:=FChangeEvent;
  Self.FDBEdit.OnKeyDown:=@FDBEditKeyDown;
  Self.FDBEdit.Width:=Self.Width - Self.FButtonWidth - 1;

  //
  Self.FButton := TBitBtn.Create(Self);
  Self.FButton.AutoSize:=False;
  Self.FButton.Parent := Self;
  Self.FButton.Height:= Self.FDBEdit.Width;
  Self.FButton.Width:= Self.FButtonWidth;
  Self.FButton.Align:=alClient;
  Self.FButton.OnClick:=@FButtonClick;

  //
  Self.OnResize:=@Self.PanelSizeChange;
  Self.DirectInput:=True;
  Self.ButtonShortCut:=KeyToShortcut( VK_Down , [ssAlt]);
end;

function TF1DBEditButton.ExecuteAction(AAction: TBasicAction): Boolean;
begin
  Result:= Self.FDBEdit.ExecuteAction(AAction);
end;

function TF1DBEditButton.UpdateAction(AAction: TBasicAction): Boolean;
begin
  Result := Self.FDBEdit.UpdateAction(AAction);
end;


procedure Register;
begin
  {$I f1dbeditbutton_icon.lrs}
  RegisterComponents('Data Controls',[TF1DBEditButton]);
end;

end.
